﻿using System;
using System.Collections.ObjectModel;
using System.Reflection;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using LifeLogApp.Classes;
using LifeLogApp.Model;
using RestSharp;

namespace LifeLogApp.ViewModel
{
    public partial class UploadViewModel : ObservableObject
    {
        Serilog.ILogger logger;
        MainViewModel mainViewModel;

        public UploadViewModel(Serilog.ILogger logger, MainViewModel mainViewModel)
        {
            this.mainViewModel = mainViewModel;

            ProjectOrFolderList = new ObservableCollection<string>();
            var pl = mainViewModel.ProjectList;

            var client = new RestClient($"https://my.unipoint.se/LifeLog_Dev/api/project/{mainViewModel.User}/list");
            client.Timeout = -1;
            var request = new RestRequest(Method.GET);
            request.AddHeader("ApiKey", "xyz");

            var fullUrl = client.BuildUri(request);
            logger.Debug("FullUrl={FullUrl}", fullUrl);

            IRestResponse response = client.Execute(request);
            logger.Debug("Response={StatusCode}, {Content}", response.StatusCode, response.Content);

            if (response.IsSuccessful)
            {
                logger.Verbose(
                    "{Method}: Projets downloaded OK",
                    AppHelper.BuildFullMethodName(MethodBase.GetCurrentMethod()));

                var projectList = Newtonsoft.Json.JsonConvert.DeserializeObject<LifeLogProjects>(response.Content);
                mainViewModel.ProjectList = projectList;
                foreach (var p in projectList.Projects)
                {
                    ProjectOrFolderList.Add(p.MoveToFolder);
                }
            }
            else
            {
                logger.Error("");
            }
            logger.Information("");
        }

        [ObservableProperty]
        ObservableCollection<string> projectOrFolderList;

        [ObservableProperty]
        string selectedProjectOrFolder;

        [ObservableProperty]
        string tag;

        [ObservableProperty]
        string what;

        [RelayCommand]
        async void Upload()
        {
            var capturedEvents = mainViewModel.CapturedEvents.ToList();

            foreach (var item in capturedEvents)
            {
                item.FolderOrProject = selectedProjectOrFolder;
                //item.Tags = new List<string>() { Tag };
                item.WhatOrHowOrWhyOrEventOrAction = What;

                var json = Newtonsoft.Json.JsonConvert.SerializeObject(item, Newtonsoft.Json.Formatting.Indented);
                var jsonFileName = Path.Combine(AppHelper.AppPathEventsUpload, Path.GetFileNameWithoutExtension(item.LocalMediaFullFileName) + ".json");
                await File.WriteAllTextAsync(jsonFileName, json);

                var newUploadFileName = Path.Combine(AppHelper.AppPathEventsUpload, Path.GetFileName(item.LocalMediaFullFileName));
                File.Move(item.LocalMediaFullFileName, newUploadFileName);

                mainViewModel.CapturedEvents.Remove(item);
            }

            mainViewModel.SaveCapturedEventsToSettings();
            mainViewModel.LoadCapturedEventsFromSettings();

            mainViewModel.RaiseOnPropertyChangedUploadFiles();

            await Shell.Current.GoToAsync("..");
        }

    }
}

